# -*- coding: utf-8 -*-
from django.core.management.base import BaseCommand, CommandError

from municipios.utils.ibge import convert_shapefile

class Command(BaseCommand):
    args = '<IBGE shapefile>'
    help = u'Importa shapefile do IBGE para os modelos UF e Municipio'
    
    def handle(self, *args, **options):
        if not args:
            raise CommandError('Arquivo Shapefile do IBGE não especificado!')
        print "Convertendo '%s':" % args[0]
        convert_shapefile(args[0])


# -*- coding: utf-8 -*-
from django.shortcuts import render_to_response
from django.template import RequestContext

from models import Municipio
from forms import FormMunicipio


def municipios_ajx(request, uf):
    municipio_list = Municipio.objects.filter(uf=uf).order_by('nome')

    return render_to_response("municipios/municipios_options.html",
                              {"municipio_list": municipio_list},
                              context_instance=RequestContext(request))
def teste(request):
    form = FormMunicipio()
    return render_to_response('municipios/teste.html',
                              {'form':form,},
                               context_instance=RequestContext(request),)



# -*- coding: utf-8 -*-
from django.contrib.gis.gdal import DataSource, SpatialReference, CoordTransform, OGRGeometry, OGRGeomType
from django.contrib.gis.gdal.geometries import Polygon
from django.template.defaultfilters import slugify
from municipios.models import UF, Municipio, MUNICIPIOS_GEO, SRID
import sys

KEEP_LOWCASE = ('de', 'da', 'das', 'do', 'dos',)

UF_SIGLAS = (
    (11, 'RO'),
    (12, 'AC'),
    (13, 'AM'),
    (14, 'RR'),
    (15, 'PA'),
    (16, 'AP'),
    (17, 'TO'),
    (21, 'MA'),
    (22, 'PI'),
    (23, 'CE'),
    (24, 'RN'),
    (25, 'PB'),
    (26, 'PE'),
    (27, 'AL'),
    (28, 'SE'),
    (29, 'BA'),
    (31, 'MG'),
    (32, 'ES'),
    (33, 'RJ'),
    (35, 'SP'),
    (41, 'PR'),
    (42, 'SC'),
    (43, 'RS'),
    (50, 'MS'),
    (51, 'MT'),
    (52, 'GO'),
    (53, 'DF')
)

UF_SIGLAS_DICT = dict([(str(c), u) for c, u in UF_SIGLAS])

def capitalize_name(s):
    res = []
    for w in s.lower().split(' '):
        if w in KEEP_LOWCASE:
            res.append(w)
        else:
            res.append(w.capitalize())
    return u" ".join(res)


def convert_shapefile(shapefilename, srid=4674):
    """
    shapefilename: considera nomenclatura de shapefile do IBGE para determinar se é UF 
                   ou Municípios.
                   ex. 55UF2500GC_SIR.shp para UF e 55MU2500GC_SIR.shp para Municípios
    srid: 4674 (Projeção SIRGAS 2000)
    """
    # /home/nando/Desktop/IBGE/2010/55MU2500GC_SIR.shp
    ds = DataSource(shapefilename)
    
    is_uf = shapefilename.upper().find('UF') != -1
    
    transform_coord = None
    if srid != SRID:
        transform_coord = CoordTransform(SpatialReference(srid), SpatialReference(SRID))

    if is_uf:
        model = UF
    else:
        model = Municipio
    
    ct = 0
    for f in ds[0]:

        # 3D para 2D se necessário
        if f.geom.coord_dim != 2:
            f.geom.coord_dim = 2
        
        # converte para MultiPolygon se necessário
        if isinstance(f.geom, Polygon):
            g = OGRGeometry(OGRGeomType('MultiPolygon'))
            g.add(f.geom)
        else:
            g = f.geom
        
        # transforma coordenadas se necessário
        if transform_coord:
            g.transform(transform_coord)

        # força 2D
        g.coord_dim = 2        
        
        kwargs = {
            'nome': capitalize_name(unicode(f.get('NOME'), 'latin1')), 
            'geom': g.ewkt
        }
        
        if is_uf:
            kwargs['id_ibge'] = f.get('GEOCODUF')
            kwargs['regiao'] = capitalize_name(unicode(f.get('REGIAO'), 'latin1'))
            kwargs['uf'] = UF_SIGLAS_DICT.get(kwargs['id_ibge'])
        else:
            kwargs['id_ibge'] = f.get('GEOCODMU')                
            kwargs['uf'] = UF.objects.get(pk=f.get('GEOCODMU')[:2])
            kwargs['uf_sigla'] = kwargs['uf'].uf 
            kwargs['nome_abreviado'] = slugify(kwargs['nome'])
            # tenta corrigir nomes duplicados, são em torno de 242 nomes repetidos
            # adicionando a sigla do estado no final
            if Municipio.objects.filter(nome_abreviado=kwargs['nome_abreviado']).count() > 0:
                kwargs['nome_abreviado'] = u'%s-%s' % (kwargs['nome_abreviado'], kwargs['uf_sigla'].lower())
            
        instance = model(**kwargs)
        instance.save()
        
        ct += 1
    
    print ct, (is_uf and "Unidades Federativas criadas" or "Municipios criados")
    
    
def update_sedes_municipais(shapefilename, srid=4618):
    # dados de um shapefile de 2001
    ds = DataSource(shapefilename)
    
    transform_coord = None
    if srid != SRID:
        transform_coord = CoordTransform(SpatialReference(srid), SpatialReference(SRID))

    ct = 0
    cta = 0
    for f in ds[0]:
        ct +=1
        cod = f.get('CODIGO')
        muns = Municipio.objects.extra(where=['CAST(id_ibge AS VARCHAR) ILIKE %s'], params=['%s%%' % cod])
        if muns:
            if len(muns) > 1:
                print "Mais de 1 municipio para", cod
                for m in muns:
                    print m.id_ibge, m
            else:
                g = f.geom
                g.srs = SpatialReference(srid)
                g.srid = srid
                if transform_coord:
                    g.transform(transform_coord)
                print g.ewkt
                muns[0].sede = g.ewkt
                muns[0].save()
                cta += 1
        else:
            print cod, "nao econtrado!"
    
    print "Atualizados", cta, "sedes"
    print "Total de", ct, "registros no shapefile"
        
if __name__ == "__main__":
    if len(sys.argv) == 1:
        print "Informe o arquivo shapefile!"
    else:
        convert_shapefile(sys.argv[1])
        









# -*- coding: utf-8 -*-
from django.conf import settings

MUNICIPIOS_GEO = getattr(settings,'MUNICIPIOS_GEO', False)

if MUNICIPIOS_GEO:
    from django.contrib.gis.db import models
else:
    from django.db import models

SRID = None
if MUNICIPIOS_GEO:
    SRID = getattr(settings,'MUNICIPIOS_SRID', '900913')
    
    
    
class UF(models.Model):
    """
    Unidades Federativas (Estados) do Brasil
    """
    id_ibge = models.IntegerField(primary_key=True)
    uf = models.CharField(max_length=2)
    nome = models.CharField(max_length=20)
    regiao = models.CharField(max_length=20)
    if MUNICIPIOS_GEO:
        geom = models.MultiPolygonField(srid=SRID, null=True, blank=True)

    def __unicode__(self):
        return self.nome
    
         
class Municipio(models.Model):
    """
    Municípios do Brasil
    """
    id_ibge = models.IntegerField(primary_key=True)
    nome = models.CharField(max_length=150)
    nome_abreviado = models.CharField(max_length=150, blank=True, null=True)    
    uf = models.ForeignKey(UF)
    uf_sigla = models.CharField(max_length=2)
    if MUNICIPIOS_GEO: 
        sede = models.PointField(srid=SRID, null=True, blank=True)
        geom = models.MultiPolygonField(srid=SRID, null=True, blank=True)
            
    def __unicode__(self):
        return u'%s - %s' % (self.nome, self.uf_sigla)
    



from django.conf.urls.defaults import patterns, include, url

from views import municipios_ajx
urlpatterns = patterns('',
    url(r'^ajax/municipios/(?P<uf>\w\w)/$', municipios_ajx, name='ajax-municipios'),   
    url(r'^teste/', 'municipios.views.teste', name='munuf'),
    

)



# -*- coding: utf-8 -*-
from django.forms.widgets import Widget, TextInput, Select, MultiWidget, HiddenInput, Input
from django.conf import settings
from django.core.urlresolvers import reverse
from django.utils.safestring import mark_safe

from municipios.models import Municipio, UF

class SelectMunicipioWidget(Widget):
    """
    Widget to render a <select> with the UFs (Brazillian states)
    and a <select> with the actual Municipio depending on the state
    selected
    """
        
    def render(self, name, value, attrs=None):
        if value is None: value = ''
        attrs = attrs or {}
        if attrs:
            self.attrs.update(attrs)
        
        output = []
        uf_val = ''
        uf_choices = [('','--')]+[(uf.pk,uf.uf) for uf in UF.objects.order_by('uf')]
        uf_select = Select(choices=uf_choices)
        municipio_select = Select(choices=(('','--'),))
        
        if value:
            try:
                municipio = Municipio.objects.get(pk=value)
                uf_val = municipio.uf
                mun_choices = [(m.id, m.nome)for m in Municipio.objects.filter(uf=uf_val).order_by('nome')]
                municipio_select = Select(choices=[('','- Selecione -')]+mun_choices)
            except Municipio.DoesNotExist:
                pass
        uf_attrs = self.attrs.copy()
        uf_attrs.update({"id":'%s_uf' % name, "onchange":"changeUF(this);"})
        select_html = uf_select.render('%s_uf' % name, uf_val, attrs=uf_attrs)
        required = False
        if 'class' in self.attrs:
            required = self.attrs['class'] == 'required'
        output.append(u'<div class="field"><label%s>UF</label><br />%s</div>' % (required and ' class="required"' or '', select_html))
        
        munic_attrs = self.attrs.copy()
        munic_attrs['style']="width:250px;"
        select_html = municipio_select.render(name, value, munic_attrs)
        output.append(u'<div class="field"><label%s>Município</label><br />%s</div>' % (required and ' class="required"' or '', select_html))
        return mark_safe(u'\n'.join(output))
    
    class Media:
        js = ( 'js/municipio.js',)




# -*- coding: utf-8 -*-
from setuptools import setup, find_packages
 
setup(
    name='django-municipios',
    version='0.7',
    description='Aplicação plugável Django com modelos e widgets para os Municípios Brasileiros',
    long_description=open('README.rst').read(),
    author='ZNC Sistemas',
    author_email='contato@znc.com.br',
    maintainer='ZNC Sistemas',
    maintainer_email='contato@znc.com.br',
    url='https://github.com/znc-sistemas/django-municipios',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        'Framework :: Django',
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: JavaScript',
        'Topic :: Utilities',
        'Natural Language :: Portuguese (Brazilian)'
    ],
)
